"use strict";
/**
 * @author Ben Siebert <ben@mctzock.de>
 * @copyright (c) 2018-2021 Ben Siebert. All rights reserved.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
exports.__esModule = true;
exports.BlocklyCompiler = void 0;
var Blockly = __importStar(require("blockly"));
var BlocklyCompiler = /** @class */ (function () {
    function BlocklyCompiler() {
        this.compiledBlocks = [];
    }
    /**
     * Compiles the current blockly workspace
     * @return The compiled code
     */
    BlocklyCompiler.prototype.compile = function () {
        var _this = this;
        var code = "";
        Blockly.getMainWorkspace()
            .getAllBlocks(true)
            .forEach(function (block) {
            // @ts-ignore
            if (!_this.compiledBlocks.includes(block)) {
                code += _this.compileBlock(block);
            }
        });
        // remove empty lines
        code = code.replace(/^\s*[\r ]*\n/gm, "");
        return code;
    };
    /**
     * Compiles a single block given by the compile method
     * @param block The block to be compiled
     */
    BlocklyCompiler.prototype.compileBlock = function (block) {
        this.compiledBlocks.push(block);
        var code = "";
        var b = block.getInputTargetBlock("STATEMENT");
        if (b) {
            code += this.getBlockTabs(block, true) + this.getBlockText(block) + "\n";
            do {
                this.compiledBlocks.push(b);
                code += this.getBlockTabs(b, true) + this.compileBlock(b) + "\n";
                /* eslint-disable  no-cond-assign */
            } while ((b = b.getNextBlock()));
        }
        else {
            code += this.getBlockTabs(block, true) + this.getBlockText(block) + "\n";
        }
        return code;
    };
    /**
     * Calculates the TABS (\t) for a given block
     * @param block The block which should be calculated
     * @param log Log the TABS (\t) to the console DEFAULT = false. DEBUGGING ONLY
     * @param b1 Ignore this boolean, idiot. Just kidding, basically I created it for debugging and it now has no sense anymore
     */
    BlocklyCompiler.prototype.getBlockTabs = function (block, log, b1) {
        if (log === void 0) { log = false; }
        if (b1 === void 0) { b1 = true; }
        var tabs = "";
        var currentBlock = block;
        if (block.getParent() != null && b1) {
            var neededTabs = -2;
            neededTabs =
                neededTabs +
                    this.getBlockTabs(block.getParent(), log, true)
                        .length -
                    1;
            if (block.getParent().getInputTargetBlock("STATEMENT"))
                neededTabs++;
            for (var i = 0; i < neededTabs; i++) {
                tabs += "    ";
            }
        }
        while (currentBlock.getParent() != null) {
            var b = currentBlock.getParent().getInputTargetBlock("STATEMENT");
            if (this.getBlockText(currentBlock.getParent()).includes("Methode"))
                tabs += "    ";
            if (b) {
                do {
                    if (b === block)
                        tabs += "    ";
                    /* eslint-disable  no-cond-assign */
                } while ((b = b.getNextBlock()));
            }
            currentBlock = currentBlock.getParent();
        }
        return tabs;
    };
    /**
     * Actually compiles a block
     * @param block The block to be compiled
     */
    BlocklyCompiler.prototype.getBlockText = function (block) {
        var c = "";
        block.inputList.forEach(function (i) {
            i.fieldRow.forEach(function (r) {
                if (r.getValue() != "Start") {
                    c += r.getValue() + " ";
                }
            });
        });
        return c;
    };
    return BlocklyCompiler;
}());
exports.BlocklyCompiler = BlocklyCompiler;
